// Copyright (C) 2003 Mooffie <mooffie@typo.co.il>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111, USA.

#include <config.h>

#include "menus.h"

#define STT_AUTOINDENT	    1001
#define STT_AUTOJUSTIFY	    1002
#define STT_ALTKBD	    1003
#define STT_SMRT	    1004
#define STT_ARABICSHAPING   1005
#define STT_FORMATMARKS	    1006
#define STT_CURSORREPORT    1007
#define STT_SPELLERLOADED   1008
#define STT_READONLY	    1009
#define STT_BIGCURSOR	    1010
#define STT_GRAPHBOXES	    1011
#define STT_KEYFORKEYUNDO   1012
#define STT_BIDI	    1013

#define STT_EOPUNIX	    5001
#define STT_EOPDOS	    5002
#define STT_EOPMAC	    5003
#define STT_EOPUNICODE	    5004

#define STT_RTLNSMASCII	    5010
#define STT_RTLNSMASIS	    5011
#define STT_RTLNSMHIDE	    5012

#define STT_MQFASCII	    5020
#define STT_MQFHIGHLIGHT    5021
#define STT_MQFASIS	    5022

#define STT_WRAPWSPACE	    5030
#define STT_WRAPBREAK	    5031
#define STT_WRAPOFF	    5032

#define STT_DIRALGOUNICODE  5040
#define STT_DIRALGOFRTL	    5041
#define STT_DIRALGOFLTR	    5042
#define STT_DIRALGOCONTEXT  5043

#define STT_SCRLBRNONE	    5060
#define STT_SCRLBRLEFT	    5061
#define STT_SCRLBRRIGHT	    5062

PulldownMenu FileMenu = {
    { "load_file", N_("~Open...") },
    { "insert_file",  N_("~Insert file...") },
    { "-----------" },
    { "save_file", N_("~Save") },
    { "save_file_as", N_("Save ~As...") },
    { "write_selection_to_file", N_("~Write selection to...") },
    { "-----------" },
    { "change_directory", N_("Ch~dir...") },
    { "quit", N_("~Quit") },
    { NULL }
};

PulldownMenu MovementMenu = {
    { "move_end_of_buffer", N_("Move to the ~end of the buffer") },
    { "move_beginning_of_buffer", N_("Move to the ~start of the buffer") },
    { "go_to_line", N_("Go to a specific ~line") },
    { "-----------" },    
    { "key_left", N_("Move a character left") },
    { "key_right", N_("Move a character right") },
    { "move_previous_line", N_("Move to the previous line") },
    { "move_next_line", N_("Move to the next line") },
    { "move_forward_page", N_("Move to the next page") },
    { "move_backward_page", N_("Move to the previous page") },
    { "move_forward_char", N_("Move forward a character") },
    { "move_backward_char", N_("Move back a character") },
    { "move_beginning_of_line", N_("Move to the start of the current line") },
    { "move_end_of_line", N_("Move to the end of the current line") },
    { NULL }
};

PulldownMenu DeletionMenu = {
    { "delete_backward_char", N_("Delete the previous character") },
    { "delete_forward_char", N_("Delete the character the cursor is on") },
    { "delete_forward_word", N_("Delete to the end of the current or next word") },
    { "delete_end_of_paragraph", N_("Delete [to] end of paragraph") },
    { "delete_paragraph", N_("Delete the current paragraph") },
    { NULL }
};

PulldownMenu EditMenu = {
    { "undo", N_("~Undo") },
    { "redo", N_("~Redo") },
    { "toggle_key_for_key_undo", N_("Toggle ~key-for-key undo"), STT_KEYFORKEYUNDO },
    { "-----------" },
    { "copy", N_("~Copy") },
    { "cut",  N_("Cu~t") },
    { "paste",N_("~Paste") },
    { "toggle_primary_mark", N_("Start/cancel selection") },
    { "-----------" },
    { "", N_("~Movement"), 0, MovementMenu },
    { "", N_("~Deletion"), 0, DeletionMenu },
    { "toggle_read_only", N_("Toggle read-only status of buffer"), STT_READONLY },
    { "-----------" },
    { "search_forward", N_("~Search...") },
    { "search_forward_next", N_("Search ~next") },
    { "-----------" },
    { "toggle_auto_justify", N_("Enable/disable auto-~justify"), STT_AUTOJUSTIFY },
    { "justify", N_("Justify the current or next paragraph") },
    { "change_justification_column", N_("Change justify column...") },
    { "-----------" },
    { "toggle_auto_indent", N_("Enable/disable auto-~indent"), STT_AUTOINDENT },
    { "change_tab_width", N_("Change the TAB character size...") },
    { NULL }
};

PulldownMenu EolMenu = {
    { "set_eops_unix",	    N_("~Unix (LF)"),		    STT_EOPUNIX },
    { "set_eops_dos",	    N_("~DOS (CR,LF)"),		    STT_EOPDOS },
    { "set_eops_mac",	    N_("~Macintosh (CR)"),	    STT_EOPMAC },
    { "set_eops_unicode",   N_("U~nicode PS (U+2029)"),	    STT_EOPUNICODE },
    { NULL }
};

PulldownMenu EncodingsMenu = {
    { "menu_set_encoding_utf_8",	N_("~UTF-8") },
    { "-----------" },
    { "menu_set_encoding_iso_8859_8",	N_("(Hebrew) ~ISO-8859-8") },
    { "menu_set_encoding_cp_1255",	N_("(Hebrew) ~Windows-1255") },
    { "menu_set_encoding_mac_hebrew",	N_("(Hebrew) ~MacHebrew") },
    { "menu_set_encoding_cp_862",	N_("(Hebrew) IBM-862 (~Dos)") },
    { "-----------" },
    { "menu_set_encoding_iso_8859_6",	N_("(Arabic) ISO-8859-6") },
    { "menu_set_encoding_cp_1256",	N_("(~Arabic) Windows-1256") },
    { "menu_set_encoding_mac_arabic",	N_("(Arabic) MacArabic") },
    { "menu_set_encoding_cp_864",	N_("(Arabic) IBM-864 (Dos)") },
    { "menu_set_encoding_mac_farsi",	N_("(Farsi) MacFarsi") },
    { "-----------" },
    { "menu_set_encoding_other",	N_("Other...") },
    { NULL }
};

PulldownMenu CharactersUtilsMenu = {
    { "show_character_code", N_("Print ~Unicode and UTF-8 value") },
    { "show_character_info", N_("Print the corresponding Unicode~Data.txt line") },
    { "insert_unicode_char", N_("~Insert a specific Unicode character") },
    { "-----------" },
    { "xxx", N_("Change the ~encoding used for saving this file"), 0, EncodingsMenu },
    { "-----------" },
    { "toggle_alt_kbd", N_("Toogle ~Hebrew keyboard emulation"), STT_ALTKBD },
    { "toggle_smart_typing", N_("Toggle ~smart-typing mode"), STT_SMRT },
    { "set_translate_next_char", N_("~Translate next character") },
    { "insert_maqaf", N_("Insert Hebrew ~maqaf") },
    { "-----------" },
    { "toggle_eops", N_("~End-of-line type"), 0, EolMenu },
    { NULL }
};

PulldownMenu DirAlgoMenu = {
    { "set_dir_algo_unicode",	N_("~Unicode's TR #9"),		STT_DIRALGOUNICODE },
    { "set_dir_algo_force_ltr",	N_("Force ~LTR"),		STT_DIRALGOFLTR },
    { "set_dir_algo_force_rtl",	N_("Force ~RTL"),		STT_DIRALGOFRTL },
    { "set_dir_algo_context",	N_("Contextual (~Geresh's own)"), STT_DIRALGOCONTEXT },
    { NULL }
};

PulldownMenu BiDiMenu = {
    { "toggle_bidi", N_("Enable/disable the ~BiDi algorithm"), STT_BIDI },
    { "toggle_dir_algo", N_("Select base-directionality ~algorithm"), 0, DirAlgoMenu },
    { NULL }
};

PulldownMenu SpellerMenu = {
    { "spell_check_all", N_("Spell check all ~document") },
    { "spell_check_forward", N_("Spell check ~from cursor onward") },
    { "spell_check_word", N_("Spell check ~word under cursor") },
    { "-----------" },
    { "load_unload_speller", N_("Explicitly ~load/unload the speller process..."), STT_SPELLERLOADED },
    { NULL }
};

PulldownMenu WrapMenu = {
    { "set_wrap_type_at_white_space",
	    N_("Wrap lines, do not break ~words"),   STT_WRAPWSPACE },
    { "set_wrap_type_anywhere",
	    N_("Wrap lines, ~break words"),	    STT_WRAPBREAK },
    { "set_wrap_type_off",
	    N_("Do ~not wrap lines"),		    STT_WRAPOFF },
    { NULL }
};

PulldownMenu rtlnsmMenu = {
    { "set_rtl_nsm_transliterated",
	    N_("Display Hebrew/Arabic points as ~ASCII characters"), STT_RTLNSMASCII },
    { "set_rtl_nsm_asis",
	    N_("Display Hebrew/Arabic points as-~is (for capable terminals)"), STT_RTLNSMASIS },
    { "set_rtl_nsm_off",
	    N_("Do ~not show Hebrew/Arabic points"), STT_RTLNSMHIDE },
    { NULL }
};

PulldownMenu MaqafMenu = {
    { "set_maqaf_display_transliterated",
	    N_("Display maqaf as ~ASCII dash"),	STT_MQFASCII },
    { "set_maqaf_display_asis",
	    N_("Display the maqaf as-~is (for capable terminals)"),
						STT_MQFASIS },
    { "set_maqaf_display_highlighted",
	    N_("~Highlight the maqaf"),		STT_MQFHIGHLIGHT },
    { NULL }
};

PulldownMenu ScrollbarMenu = {
    { "menu_set_scrollbar_none",    N_("~Off"),		STT_SCRLBRNONE },
    { "menu_set_scrollbar_left",    N_("~Left"),	STT_SCRLBRLEFT },
    { "menu_set_scrollbar_right",   N_("~Right"),	STT_SCRLBRRIGHT },
    { NULL }
};

PulldownMenu DisplayMenu = {
    { "xxx", N_("Change ~wrap method"), 0, WrapMenu },
    { "toggle_arabic_shaping",   N_("Toggle ~Arabic shaping"), STT_ARABICSHAPING },
    { "toggle_formatting_marks", N_("Toggle display of ~formatting marks"), STT_FORMATMARKS },
    { "xxx", N_("Change display of Hebrew/Arabic ~points"), 0, rtlnsmMenu },
    { "xxx", N_("Change display of Hebrew ~maqaf"), 0, MaqafMenu },
    { "-----------" },
    { "xxx", N_("~Scrollbar"), 0, ScrollbarMenu },
    { "toggle_cursor_position_report", N_("Toggle display of cursor p~osition"), STT_CURSORREPORT },
    { "change_scroll_step", N_("Change the scroll step") },
    { "refresh_and_center", N_("~Repaint the terminal and center the line") },
    { "toggle_graphical_boxes", N_("Toggle ~graphical boxes"), STT_GRAPHBOXES },
    { "toggle_big_cursor",	N_("Toggle ~big cursor"), STT_BIGCURSOR },
    { NULL }
};

PulldownMenu HelpMenu = {
    { "help",		N_("User ~Manual") },
    { "describe_key",	N_("~Describe key...") },
    { NULL }
};

MenubarMenu mainMenu = {
    { "File", FileMenu },
    { "Edit", EditMenu },
    { "Characters", CharactersUtilsMenu },
    { "Display", DisplayMenu },
    { "BiDi", BiDiMenu },
    { "Speller", SpellerMenu },
    { "Help", HelpMenu },
    { NULL }
};

////////////////////////////// MMPopupMenu /////////////////////////////////

class MMPopupMenu : public PopupMenu {
    Editor  *editor;
    EditBox *editbox;
public:
    MMPopupMenu(Editor *aEditor, EditBox *aEditbox,	    
		PopupMenu *aParent, PulldownMenu aMnu);
protected:
    virtual void show_hint(const char *hint);
    virtual void clear_other_popups();
    virtual Dispatcher *get_primary_target();
    virtual Dispatcher *get_secondary_target();
    virtual PopupMenu *create_popupmenu(PopupMenu *aParent, PulldownMenu mnu);
    virtual bool get_item_state(int id);
};

MMPopupMenu::MMPopupMenu(Editor *aEditor, EditBox *aEditbox,	
			 PopupMenu *aParent, PulldownMenu aMnu)
    : PopupMenu(aParent, NULL)
{
    editor  = aEditor;
    editbox = aEditbox;
    init(aMnu);
}

void MMPopupMenu::show_hint(const char *hint)
{
    editor->show_hint(hint);
}

void MMPopupMenu::clear_other_popups()
{
    editor->refresh(true);
}

Dispatcher *MMPopupMenu::get_primary_target()
{
    return editor;
}

Dispatcher *MMPopupMenu::get_secondary_target()
{
    return editbox;
}

PopupMenu *MMPopupMenu::create_popupmenu(PopupMenu *aParent, PulldownMenu mnu)
{
    return new MMPopupMenu(editor, editbox, aParent, mnu);
}

bool MMPopupMenu::get_item_state(int id)
{
    switch (id) {
    case STT_AUTOINDENT:    return editbox->is_auto_indent();
    case STT_AUTOJUSTIFY:   return editbox->is_auto_justify();
    case STT_ALTKBD:	    return editbox->is_alt_kbd_active();
    case STT_SMRT:	    return editbox->is_smart_typing();
    case STT_ARABICSHAPING: return terminal::do_arabic_shaping;
    case STT_FORMATMARKS:   return editbox->has_formatting_marks();
    case STT_CURSORREPORT:  return editor->is_cursor_position_report();
    case STT_READONLY:	    return editbox->is_read_only();
    case STT_BIGCURSOR:	    return editor->is_big_cursor();
    case STT_GRAPHBOXES:    return terminal::graphical_boxes;
    case STT_KEYFORKEYUNDO: return editbox->is_key_for_key_undo();
    case STT_BIDI:	    return editbox->is_bidi_enabled();

    case STT_EOPUNIX:	    return editbox->get_dominant_eop() == eopUnix;
    case STT_EOPDOS:	    return editbox->get_dominant_eop() == eopDOS;
    case STT_EOPMAC:	    return editbox->get_dominant_eop() == eopMac;
    case STT_EOPUNICODE:    return editbox->get_dominant_eop() == eopUnicode;

    case STT_RTLNSMASCII:  return editbox->get_rtl_nsm_display() == EditBox::rtlnsmTransliterated;
    case STT_RTLNSMASIS:   return editbox->get_rtl_nsm_display() == EditBox::rtlnsmAsis;
    case STT_RTLNSMHIDE:   return editbox->get_rtl_nsm_display() == EditBox::rtlnsmOff;

    case STT_MQFASCII:	    return editbox->get_maqaf_display() == EditBox::mqfTransliterated;
    case STT_MQFHIGHLIGHT:  return editbox->get_maqaf_display() == EditBox::mqfHighlighted;
    case STT_MQFASIS:	    return editbox->get_maqaf_display() == EditBox::mqfAsis;

    case STT_WRAPOFF:	    return editbox->get_wrap_type() == EditBox::wrpOff;
    case STT_WRAPBREAK:	    return editbox->get_wrap_type() == EditBox::wrpAnywhere;
    case STT_WRAPWSPACE:    return editbox->get_wrap_type() == EditBox::wrpAtWhiteSpace;

    case STT_DIRALGOUNICODE:    return editbox->get_dir_algo() == algoUnicode;
    case STT_DIRALGOFRTL:	return editbox->get_dir_algo() == algoForceRTL;
    case STT_DIRALGOFLTR:	return editbox->get_dir_algo() == algoForceLTR;
    case STT_DIRALGOCONTEXT:    return editbox->get_dir_algo() == algoContext;

    case STT_SPELLERLOADED:	return editor->is_speller_loaded();

    case STT_SCRLBRNONE:    return editor->get_scrollbar_pos() == Editor::scrlbrNone;
    case STT_SCRLBRLEFT:    return editor->get_scrollbar_pos() == Editor::scrlbrLeft;
    case STT_SCRLBRRIGHT:   return editor->get_scrollbar_pos() == Editor::scrlbrRight;

    default:
	return false;
    }
}

/////////////////////////////// MMMenubar //////////////////////////////////

class MMMenubar : public Menubar {
    Editor  *editor;
    EditBox *editbox;
public:
    MMMenubar(Editor *, EditBox *);
    virtual void refresh_screen();
    virtual PopupMenu *create_popupmenu(PulldownMenu mnu);
};

MMMenubar::MMMenubar(Editor *aEditor, EditBox *aEditbox)
    : Menubar()
{
    editor  = aEditor;
    editbox = aEditbox;
    init(mainMenu);
}

void MMMenubar::refresh_screen()
{
    editor->refresh(true);
}

PopupMenu *MMMenubar::create_popupmenu(PulldownMenu mnu)
{
    return new MMPopupMenu(editor, editbox, NULL, mnu);
}

////////////////////////////////////////////////////////////////////////////

Menubar *create_main_menubar(Editor *aEdtr, EditBox *aEdtbx)
{
    return new MMMenubar(aEdtr, aEdtbx);
}

